/* Sticky footer styles
-------------------------------------------------- */
html {
  position: relative;
  min-height: 100%;
}
body {
  /* EDITOR: Margin bottom by footer height, total height */
  margin-bottom: 50px;
}
#footer {
  position: absolute;
  bottom: 0;
  width: 100%;
  /* EDITOR: Set the fixed height of the footer here, total height */
  height: 50px;
  background-color: #f5f5f5;
}


/* Custom page CSS
-------------------------------------------------- */
/* Not required for template or sticky footer method. */

body > .container {
  /* EDITOR: total height */
  padding: 50px 15px 0;
}
.container .text-muted {
  /* EDITORS: margin is counted 2 times, margin-top, margin-bottom*/
  margin: 15px 0;
}

#footer > .container {
  padding-right: 15px;
  padding-left: 15px;
}

code {
  font-size: 80%;
}
