# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-01-19 15:26
# Simplified by Juanmi Taboada on 2017-14-02 11:12
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('codenerix', '0007_corporateimage'),
    ]

    operations = []
