# -*- coding: utf-8 -*-
# Generated by Django 1.10.6 on 2017-04-05 08:15
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('codenerix', '0011_delete_corporateimage'),
    ]

    operations = [
        migrations.AlterField(
            model_name='log',
            name='action_flag',
            field=models.PositiveSmallIntegerField(choices=[(1, 'A\xf1adir'), (2, 'Modificar'), (3, 'Eliminar')], verbose_name='Acci\xf3n'),
        ),
    ]
